#include <string.h>
#include <stdint.h>
#include <pthread.h>

#include "ik_arm.h"
#include "serial.h"

static int inited = 0;
static unsigned int adc_data[NUM_ADCS];
static pthread_mutex_t adc_mutex;
static pthread_t adc_thread;

static void *adc_thread_main(void *param)
{
	int ch, i;
	unsigned int data[NUM_ADCS];
	
	while (1)
	{
		/* First header byte */
		ch = ser_receive();
		if (ch != 0x9c)
			continue;
		
		/* Second header byte */
		ch = ser_receive();
		if (ch != 0x55)
			continue;
		
		for (i = 0; i < 6; i++)
		{
			uint8_t l, h;
			
			l = ser_receive();
			h = ser_receive();
			data[i] = (h << 8) | l;
		}
		
		pthread_mutex_lock(&adc_mutex);
		memcpy(adc_data, data, sizeof(data));
		pthread_mutex_unlock(&adc_mutex);
	}
}

int arm_init(const char *device)
{
	if (inited)
		return 0;
	
	if (!ser_init(device))
		return 0;
	
	pthread_mutex_init(&adc_mutex, 0);
	
	if (pthread_create(&adc_thread, 0, adc_thread_main, 0))
	{
		ser_shutdown();
		
		return 0;
	}
	
	inited = 1;
	
	return 1;
}

void arm_shutdown(void)
{
	if (!inited)
		return;
	
	pthread_cancel(adc_thread);
	ser_shutdown();
	inited = 0;
}

void arm_set_motors(int *data)
{
	int i;
	
	ser_send(0xc9);
	ser_send(0xaa);
	
	for (i = 0; i < NUM_MOTORS; i++)
	{
		if (data[i] >= 0)
		{
			ser_send(data[i] | 0x20);
		} else {
			ser_send(-data[i]);
		}
	}
}

void arm_get_adcs(unsigned int *data)
{
	pthread_mutex_lock(&adc_mutex);
	memcpy(data, adc_data, sizeof(adc_data));
	pthread_mutex_unlock(&adc_mutex);
}
