#include <stdio.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/ioctl.h>

#include "serial.h"

int ser_handle = -1;

int ser_init(const char *device)
{
	struct termios attr;

	ser_handle = open(device, O_RDWR | O_NOCTTY | O_NONBLOCK);
	if (ser_handle < 0)
		return 0;

	tcgetattr(ser_handle, &attr);
	cfsetispeed(&attr, B115200);
	cfsetospeed(&attr, B115200);
	cfmakeraw(&attr);
	attr.c_cflag |= CLOCAL | CREAD;
	attr.c_iflag &= ~IXOFF & ~IXANY;
	tcsetattr(ser_handle, TCSANOW, &attr);

	fcntl(ser_handle, F_SETFL, 0);

	return 1;
}

void ser_shutdown()
{
	if (ser_handle >= 0)
	{
		close(ser_handle);
		ser_handle = -1;
	}
}

void ser_send(unsigned char ch)
{
	write(ser_handle, &ch, 1);
}

void ser_send_buf(unsigned char *buf, unsigned int len)
{
	write(ser_handle, buf, len);
}

int ser_receive()
{
	unsigned char ch;

	if (read(ser_handle, &ch, 1) != 1)
		return -1;

	return ch;
}
