#include <avr/io.h>
#include <avr/interrupt.h>

#include "uart.h"

unsigned char tx_buf[16];

/* The position of the next byte to be written into the buffer */
uint8_t tx_write_pos;

/* The position of the next byte to be read from the buffer */
uint8_t tx_send_pos;

ISR(UART_TX_vect)
{
	/* Are there still bytes left to send? */
	if (tx_write_pos != tx_send_pos)
	{
		/* Send this byte */
		UDR = tx_buf[tx_send_pos++];
		
		if (tx_send_pos >= sizeof(tx_buf))
		{
			tx_send_pos = 0;
		}
	}
}

void uart_init()
{
	UCSRB = (1 << RXCIE) | (1 << TXCIE) | (1 << RXEN) | (1 << TXEN);
	UBRR = 1;			/* 115200 baud with a 3.6864MHz clock */
	
	tx_write_pos = 0;
	tx_send_pos = 0;
}

void uart_send(uint8_t byte)
{
	cli();
	if (UCSRA & (1 << UDRE))
	{
		/* No byte currently being sent - send immediately */
		UDR = byte;
	} else {
		/* A byte is being sent, so add this one to the buffer */
		tx_buf[tx_write_pos++] = byte;
		
		if (tx_write_pos >= sizeof(tx_buf))
		{
			tx_write_pos = 0;
		}
	}
	sei();
}
